#!/bin/sh
# dist-hook.sh -- sets the version and date of documentation files

# Copyright 2012-2016 University of New Hampshire Interoperability Laboratory
# All rights reserved.
#
# This file is part of the UNH EXS implementation of the
# Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
# Document Number C050 published by The Open Group, January 2005,
# and copyright 2005 by The Open Group.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING.GPL in the main directory of this source tree, or the
# BSD license, available from the file COPYING.BSD in the main directory
# of this source tree.
#
# This material is based upon work supported by the National Science
# Foundation under Grants No. OCI-1127228 and DGE-0913620.
#
# Any opinions, findings, and conclusions or recommendations
# expressed in this material are those of the authors and do not
# necessarily reflect the views of the National Science Foundation.

# This script is designed to run from the dist-hook target of Makefile.am
# It sets the date and version for all man pages and other documentation files

top_srcdir=$1
shift
top_distdir=$1
shift
VERSION=$1
shift

if expr x${top_distdir} : x/ >/dev/null; then
	# this should never happen, but if top_distdir is already an absolute
	# path, we don't need to do anything
	:
else
	# distdir is given relative to builddir, so we need to make it an
	# absolute path so we can still find it when we change to top_srcdir
	top_distdir=$(pwd)/${top_distdir}
fi

if [ ! -d ${top_srcdir}/.git ]; then
	# Do not do anything if we aren't running from a git tree
	# This ensures that recursive "make dist" behaves normally
	exit 0
fi

# To use git to fetch log info we need to be in the top-level source directory
cd ${top_srcdir}

. build-aux/git-date-order-arg.sh

for f in "$@"; do
	d=$(git log --no-merges ${DATE_ORDER} -n 1 --date=short --pretty=format:%ad -- $f)
	chmod u+w ${top_distdir}/$f
	sed -i -e "s/@VERSION@/${VERSION}/" -e "s/@DATE@/$d/" ${top_distdir}/$f
done
