/*
 * Copyright 2007-2017 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_init_setup.h"

#define INIT_TESTS 3

static char *all_init_tests[INIT_TESTS] = {
	"exs_init(INVALID_VERSION)",
	"exs_init(VERSION)",
	"exs_init(VERSION) [exs_init already]"
};

static void *test_init(void *test_void_ptr)
{
	param *p = (param *) test_void_ptr;
	int test = p->test_number;
	int ret;

	/* ENOTSUP: invalid version number */
	if (test == 1) {
		p->test_value = "ENOTSUP";
		p->test_name = all_init_tests[test - 1];
		setup_fault_catcher(p, 0);
		ret = exs_init(-1);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    ENOTSUP);
	}

	/* OK_EXS_INIT */
	else if (test == 2) {
		p->test_value = "OK_EXS_INIT";
		p->test_name = all_init_tests[test - 1];
		setup_fault_catcher(p, 0);
		if ((ret = ok_exs_init(p)) < 0) {
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, ENOTSUP);
		} else {
			print_macro(test, p->test_value, p->test_name, ret,
				    0, 0);
		}
	}

	/* EALREADY: exs_init has already been called */
	else if (test == 3) {
		p->test_value = "EALREADY";
		p->test_name = all_init_tests[test - 1];
		setup_fault_catcher(p, 0);
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		int ret;
		ret = exs_init(EXS_VERSION);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EALREADY);
	}

    /** test number not valid **/
	else if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}

	return NULL;
}

int main(int argc, char *argv[])
{
	execute(argc, argv, test_init, all_init_tests, INIT_TESTS);
}
