/*
 * Copyright 2007-2017 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_init_setup.h"
#include "setup/exs_qcreate_setup.h"
#include "setup/exs_qdequeue_setup.h"

#define QDEQUEUE_TESTS 5

static char *all_qdequeue_tests[QDEQUEUE_TESTS] = {
	"exs_qdequeue(qh, evtvec, evtvec_cnt, to) [no exs_init]",
	"exs_qdequeue(INVALID_QH, evtvec, evtvec_cent, to)",
	"exs_qdequeue(qh, evtvec, ZERO_EVTVEC_CNT, to) [evtvec_cnt < 1]",
	"",
	"exs_qdequeue(qh, evtvec, evtvec_cent, to)"
};

static void *test_qdequeue(void *test_void_ptr)
{
	param *p = (param *) test_void_ptr;
	int test = p->test_number;

	/* EPERM: exs_init has not been called */
	if (test == 1) {
		p->test_value = "EPERM";
		p->test_name = all_qdequeue_tests[test - 1];
		setup_fault_catcher(p, 0);
		int ret =
		    exs_qdequeue(p->qh, p->evtvec, p->evtvec_cnt, p->to);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EPERM);
	}

	/* EINVAL: qhandle is invalid */
	else if (test == 2) {
		p->test_value = "EINVAL";
		p->test_name = all_qdequeue_tests[test - 1];
		setup_fault_catcher(p, 0);
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		int ret =
		    exs_qdequeue(p->qh, p->evtvec, p->evtvec_cnt, p->to);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* EINVAL: evtvec_cnt is less than 1 */
	else if (test == 3) {
		p->test_value = "EINVAL";
		p->test_name = all_qdequeue_tests[test - 1];
		setup_fault_catcher(p, 0);
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		if (ok_exs_qcreate(p) < 0)
			exit(EXIT_FAILURE);
		int ret = exs_qdequeue(p->qh, p->evtvec, 0, p->to);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* EINVAL: evtvec_cnt is greater than EXS_EVTVEC_MAX */
	else if (test == 4) {	//integer overflow

	}

	/* OK_EXS_QDEQUEUE */
	else if (test == 5) {
		if (p->mode > 0 && 0) {	//asynchronous TODO
			p->test_value = "OK_EXS_QDEQUEUE";
			p->test_name = all_qdequeue_tests[test - 1];
			setup_fault_catcher(p, 0);

			if (ok_exs_init(p) < 0)
				exit(EXIT_FAILURE);

			if (ok_exs_qcreate(p) < 0)
				exit(EXIT_FAILURE);
			int ret = ok_exs_qdequeue(p);

			if (ret >= 0) {
				print_macro(test, p->test_value,
					    p->test_name, ret, 0, 0);
			} else {
				print_macro(test, p->test_value,
					    p->test_name, ret, -1,
					    ENOTSUP);
			}
		}
	}

	/** test number not valid **/
	else if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}

	return NULL;
}

int main(int argc, char *argv[])
{				//arv[1] first arg from test_all.sh
	execute(argc, argv, test_qdequeue, all_qdequeue_tests, QDEQUEUE_TESTS);
}
